/******************************************************************************
* Copyright (c) 2018, Sylvain Corlay and Johan Mabille, Wolf Vollprecht and   *
* Martin Renou                                                                *
*                                                                             *
* Distributed under the terms of the BSD 3-Clause License.                    *
*                                                                             *
* The full license is in the file LICENSE, distributed with this software.    *
*******************************************************************************/

#ifndef XWEBRTC_CONFIG_CLING_HPP
#define XWEBRTC_CONFIG_CLING_HPP

#pragma cling add_library_path("C:/bld/xwebrtc_1545945664405/_h_env/Library/lib")
#pragma cling load("libxwebrtc")

#endif
